const dtsGenerator = require('dtsgenerator');
const fs = require('fs');

const schema = require('./schema.json');

/**
 * Alters the OpenAPI 2.0 schema from demandware to make it more generalized.
 *
 * - Removes "c_*" attributes from the schema definitions.
 *   - Those attributes are custom extensions and may not be valid for all instances.
 * - Changes "country_code" attributes to be a plain string
 *
 * @param {object} schema - OpenAPI 2.0 schema
 */
function alterSchema(schema) {
  for (const definitionName in schema.definitions) {
    const definition = schema.definitions[definitionName];

    for (const property in definition.properties) {
      if (property.indexOf('c_') === 0) {
        delete definition.properties[property];

      } else if (property === 'country_code') {
        delete definition.properties[property].enum;
      }
    }
  }

  return schema;
}

/**
 * Renames global namespace to "SFCC" instead of the generic "Definitions"
 *
 * @param {string[]} id - Schema ID
 */
function typeNameConvertor(id) {
  const names = dtsGenerator.DefaultTypeNameConvertor(id);
  if (names.length > 0) {
    names[0] = 'SFCC';
  }

  return names;
}

async function main() {
  const content = alterSchema(schema);

  const result = await dtsGenerator.default({
    contents: [content],
    typeNameConvertor
  });

  const output = `
/* tslint:disable */

/**
 * Auto-generated Salesforce Commerce Cloud Open Commerce API response types for v${(schema.info || {}).version}
 */

// DO NOT EDIT THIS FILE - See fssalesforce/scripts/README.md for more info

${result}
`;

  fs.writeFileSync('./types/SFCC.d.ts', output);
}

main();
